#include <stdio.h>

typedef signed char int8;

typedef struct BlockCell {
	int8 x;
	int8 y;
} BLOCK_CELL;

typedef struct blockmodel {
	int8 NumberOfModel;
	int8 BlockSize;
	int8 BlockInUsed;
	BLOCK_CELL *Block[8];
} BLOCK_MODEL;

#define AllBlockInUsed()  (G_BlockInUsed >= G_TotalBlocks)

void Usage(char *FName);
int  ReadMatrixData(FILE *fp);
void ShowMatrixData(void);
void ShowMatrix(BLOCK_CELL *Block, int8 BlockSize);
void ShowPos(int x, int y);
void SkipSpace(char	*buf, int *Pos);
void SkipDigital(char *buf, int *Pos);
int  SkipTillDigital(char *buf, int *Pos);
void CreateBlockModel(void);
void PrintBlockModel(BLOCK_MODEL *BlockModel);
void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize);
int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock);
int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize,BLOCK_CELL *CurPos, BLOCK_CELL *NextPos, char value);
void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos);
void TryMatrix(int HeadItem, BLOCK_CELL *CurPos);
int8 FindNextAvailableBlock(int8 CurItem);

void PrintMatrix(char *str);

int8 G_MaxRow;
int8 G_MaxCol;
int8 **G_Matrix;

int8 G_BlockInUsed;
int8 G_TotalBlocks;

int  G_TotalAnswer;

BLOCK_MODEL *G_BlockModel;

BLOCK_CELL RotateUnit[] = { {1, -1} , {-1, 1}, {-1, -1} };

//        ͻ
//            1    
//        28͹
//            4    
//        ͼ

//      (3)    (5)    (6)      (7)     (9)  	  (10)
//        ͼ             ͻ    ͹               
//                                  
//
//                (12)         (13)         (14)          (15)
//     (11)                                                     
//                                   
//                                                             
//


//                  0    1    2    3	4    5    6    7
char TanFlag[] = { ' ', 'X', 'X', '', 'X', '', '', '',
//                  8    9   10   11   12   13   14   15
                   'X', '', '', '', '', '', '', ''  };


int main(int argc, char *argv[])
{
	FILE *fp;
	BLOCK_CELL StartPos;

	if(argc != 2) {
	    Usage(argv[0]);
	    return (1);
	}
	if( (fp = fopen(argv[1], "rt")) == NULL) {
                printf("Open File: %s Error !\n",argv[1]);
                Usage(argv[0]);
                return (2);
	}

	if(ReadMatrixData(fp)) {
                //Parse Matrix Data Error !
	        fclose(fp);
	        return (3);
	}
	//ShowMatrixData();
	CreateBlockModel();

	StartPos.x = StartPos.y = 1;
	TryMatrix(0, &StartPos);
	printf("Total Answer = %d\n",G_TotalAnswer);
	return (0);
}

void Usage(char *FName)
{
	printf("\nUsage: %s <Matrix Data File> !\n ", strrchr(FName,'/')+1);
	printf("\n  Matrix Data Format :\n");
	printf("       //Matrix Size (X Y) \n");
	printf("       10 10\n");
	printf("       //How many Blocks: \n");
	printf("       2\n");
	printf("       //Each Block: BlockSize, X1 Y1, X2 Y2, X3 Y3 ....\n");
	printf("       5, 0 0, 1 0, 2 0, 3 0, 0 1\n");
	printf("       4, 0 0, 0 1, 0 2, 0 3\n\n");
}

int ReadMatrixData(FILE *fp)
{
	char buf[256];
	int  ReadStep = 0;
	int  CurPos = 0;
	int8 CurNumOfBlock = 0;
	int  LineNumber = 0;
	int8 CurNumOfCell;
        int  i, BlockSize;

	while(fgets(buf,255,fp) != NULL) {
		CurPos = 0;
		LineNumber++;
		SkipSpace(buf,&CurPos);
		if(buf[CurPos] == '/' || buf[CurPos] == '\0') continue;
		if(!isdigit(buf[CurPos])) {
			printf("ReadMatrixData(): Syntax Error");
			ShowPos(LineNumber,CurPos+1);
			return (99);
		}
		switch(ReadStep) {
		case 0:	//Read Matrix Size
			G_MaxCol = atoi(buf+CurPos);
			SkipDigital(buf,&CurPos);
			if(!SkipTillDigital(buf, &CurPos)) {
				printf("ReadMatrixData():  Parse Matrix Size Error");
				ShowPos(LineNumber,CurPos+1);
				return (1);
			}
			G_MaxRow = atoi(buf+CurPos);
			if(!(G_Matrix = (int8 **) malloc((G_MaxCol+2) * sizeof(int8 *)))) {
				printf("ReadMatrixData(): Alloc Matrix Array Fail (1)");
				ShowPos(LineNumber,CurPos+1);
				return (2);
			}
			for(i = 0 ; i < (G_MaxCol+2); i++) {
				if(!(G_Matrix[i] = (int8 *) calloc((G_MaxRow+2) , sizeof(int8)))) {
					printf("ReadMatrixData(): Alloc Matrix Array Fail (2)");
					ShowPos(LineNumber,CurPos+1);
					return (3);
				}
			}
			ReadStep++;
			break;
		case 1: //Read Total Blocks
			G_BlockInUsed =  G_TotalAnswer = 0;

			G_TotalBlocks = atoi(buf+CurPos);
			if(!(G_BlockModel = (BLOCK_MODEL *) calloc(G_TotalBlocks , sizeof(BLOCK_MODEL)))) {
				printf("ReadMatrixData(): Alloc Total Blocks Fail");
				ShowPos(LineNumber,CurPos+1);
				return (4);
			}
			ReadStep++;
			break;
		case 2:	//Read Block Cell for each Block

			//Read BlockSize
			if( (BlockSize = atoi(buf+CurPos)) <= 0) {
				printf("ReadMatrixData(): Wrong Block %d Size", CurNumOfBlock+1);
				ShowPos(LineNumber,CurPos+1);
				return (6);
			}

			//Alloc BlockSize
			G_BlockModel[CurNumOfBlock].NumberOfModel = 1;
			G_BlockModel[CurNumOfBlock].BlockSize = BlockSize;
			G_BlockModel[CurNumOfBlock].BlockInUsed = 0;

			G_BlockModel[CurNumOfBlock].Block[0] = (BLOCK_CELL *) calloc(BlockSize , sizeof(BLOCK_CELL));
			if(!G_BlockModel[CurNumOfBlock].Block[0]) {
				printf("ReadMatrixData(): Alloc Block %d Fail", CurNumOfBlock);
				ShowPos(LineNumber,CurPos+1);
				return (7);
			}

			CurNumOfCell = 0;
			do {

				SkipDigital(buf,&CurPos);
				//Skip until digital (,)
				if(!SkipTillDigital(buf, &CurPos)) {
					printf("ReadMatrixData(): Parse Block %d Error", CurNumOfBlock+1);
					ShowPos(LineNumber,CurPos+1);
					return (8);
				}

				//Read Block Cell X
				G_BlockModel[CurNumOfBlock].Block[0][CurNumOfCell].x = atoi(buf+CurPos);

				SkipDigital(buf,&CurPos);

				//Skip Space
				SkipSpace(buf,&CurPos);
				if(!isdigit(buf[CurPos]) && buf[CurPos] != '-') {
					printf("ReadMatrixData(): Syntax Error (Block %d . X%d)",CurNumOfBlock+1, CurNumOfCell+1);
					ShowPos(LineNumber,CurPos+1);
					return (10);
				}

				//Read Block Cell Y
				G_BlockModel[CurNumOfBlock].Block[0][CurNumOfCell].y = atoi(buf+CurPos);

				CurNumOfCell++;

			} while(CurNumOfCell < BlockSize);
			if(++CurNumOfBlock >= G_TotalBlocks) ReadStep++;
			break;
		case 3:
		default:
			printf("ReadMatrixData(): Too many Blocks");
			ShowPos(LineNumber,CurPos+1);
			return (12);
		}
	}// while (....)
	if(ReadStep != 3) {
		printf("ReadMatrixData(): Too few data");
		ShowPos(LineNumber+1,1);
		return (15);
	}
	return (0);	//OK
}

void ShowPos(int x, int y)
{
	printf(" at Line %d Column %d\n", x,y);
}

void SkipSpace(char	*buf, int *Pos)
{
	while(buf[*Pos] == ' ' || buf[*Pos] == '\t' || buf[*Pos] == '\n') (*Pos)++;
}

void SkipDigital(char *buf, int *Pos)
{
	while(isdigit(buf[*Pos]) || buf[*Pos] == '-') (*Pos)++;
}

int SkipTillDigital(char *buf, int *Pos)
{
	while(buf[*Pos] != '-' && !isdigit(buf[*Pos]) && buf[*Pos] != '\0') (*Pos)++;

	return (isdigit(buf[*Pos]) || (buf[*Pos] == '-'));
}

void ShowMatrixData(void)
{
	int i, k;

	printf("\n//Matrix Size (X Y) \n");
	printf("%d %d\n\n", G_MaxCol, G_MaxRow);
	printf("//How many Blocks: \n");
	printf("%d\n\n",G_TotalBlocks);
	printf("//Each Block: BlockSize, X1 Y1, X2 Y2, X3 Y3 ....\n");
	for(i = 0 ; i < G_TotalBlocks; i++) {
		printf("\n//Block %d :\n", i+1);
		ShowMatrix(G_BlockModel[i].Block[0],G_BlockModel[i].BlockSize);
		printf("%d",G_BlockModel[i].BlockSize);
		for(k =  0 ; k < G_BlockModel[i].BlockSize; k++) {
			printf(", %d %d", G_BlockModel[i].Block[0][k].x,G_BlockModel[i].Block[0][k].y);
		}
		printf("\n");
	}
}

void ShowMatrix(BLOCK_CELL *Block, int8 BlockSize)
{
	BLOCK_CELL CurPos, NextPos;
	int i;

	for(i = 1 ; i <= G_MaxCol; i++) {
		CurPos.x = i;
		CurPos.y = 1;
		if(InsertToMatrix(Block, BlockSize, &CurPos, &NextPos,1)) {
			PrintMatrix("//  ");
			DeleteFromMatrix(Block, BlockSize, &CurPos);
			return;
		}
	}
	printf("This Block can not put into Matrix !\n");
}




void CreateBlockModel(void)
{
	int        item;
	BLOCK_CELL *CurBlock;
	int8	   RotateTimes, i, j, k;

	for(item = 0 ; item < G_TotalBlocks; item++) {
		CurBlock = (BLOCK_CELL *) malloc(G_BlockModel[item].BlockSize * sizeof(BLOCK_CELL));

		//(X , Y) ==> (Y , X)
		for(i = 0 ; i < G_BlockModel[item].BlockSize; i++) {
			CurBlock[i].x = G_BlockModel[item].Block[0][i].y;
			CurBlock[i].y = G_BlockModel[item].Block[0][i].x;
		}
		BlockNormalization(CurBlock, G_BlockModel[item].BlockSize);
		RotateTimes = (InsertBlock(G_BlockModel+item,CurBlock)) ? 2 : 1;

		for(i = 0 ; i < RotateTimes; i++) {

			//Insert Rotate Block !
			for(j = 0 ; j < sizeof(RotateUnit) / sizeof(BLOCK_CELL); j++) {
				for(k = 0 ; k < G_BlockModel[item].BlockSize; k++) {
					CurBlock[k].x = G_BlockModel[item].Block[i][k].x * RotateUnit[j].x;
					CurBlock[k].y = G_BlockModel[item].Block[i][k].y * RotateUnit[j].y;
				}
				BlockNormalization(CurBlock, G_BlockModel[item].BlockSize);
				InsertBlock(G_BlockModel+item, CurBlock);
			}
		}
		free(CurBlock);
//		for(i = 0 ; i < G_BlockModel[item].NumberOfModel; i++) {
//			ShowMatrix(G_BlockModel[item].Block[i],G_BlockModel[item].BlockSize);
//		}
	}
}


void BlockCopy(BLOCK_CELL *SrcBlock, BLOCK_CELL *TagBlock, int8 BlockSize)
{
	int8 i;

	for(i = 0; i < BlockSize; i++) {
		TagBlock[i].x = SrcBlock[i].x;
		TagBlock[i].y = SrcBlock[i].y;
	}
}

void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize)
{
	BLOCK_CELL TmpBlockCell;
	int8 i, j;

	//Sort it  (compare Y then X)
	for(i = 0 ; i < (BlockSize-1); i++) {
		for(j = i+1; j < BlockSize; j++) {
			if( (Block[j].y < Block[i].y) ||
				(Block[j].y == Block[i].y  && Block[j].x < Block[i].x) )
			{
				//swapt it
				TmpBlockCell.x = Block[j].x;
				TmpBlockCell.y = Block[j].y;

				Block[j].x = Block[i].x;
				Block[j].y = Block[i].y;

				Block[i].x = TmpBlockCell.x;
				Block[i].y = TmpBlockCell.y;

			}
		}
	}

	//Set smallest cell as (0,0)

	TmpBlockCell.x = Block[0].x;
	TmpBlockCell.y = Block[0].y;

	for(i = 0; i < BlockSize; i++) {
		Block[i].x -=  TmpBlockCell.x;
		Block[i].y -=  TmpBlockCell.y;
//		printf("(%d, %d) ",Block[i].x,Block[i].y);
	}
//	printf("\n");

}

int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock)
{
	int8 i, j;

	for(i = 0 ; i < BlockModel->NumberOfModel ; i++) {
		for(j = 0 ; j < BlockModel->BlockSize; j++) {
			if((BlockModel->Block[i])[j].x != CurBlock[j].x ||
			   (BlockModel->Block[i])[j].y != CurBlock[j].y)
			{
				break;
			}
		}
		if( j >= BlockModel->BlockSize) {
			return 0;
		}
	}

	BlockModel->Block[BlockModel->NumberOfModel] = (BLOCK_CELL *) malloc(BlockModel->BlockSize * sizeof(BLOCK_CELL));
	for(i = 0 ; i < BlockModel->BlockSize; i++) {
		BlockModel->Block[BlockModel->NumberOfModel][i].x = CurBlock[i].x;
		BlockModel->Block[BlockModel->NumberOfModel][i].y = CurBlock[i].y;
	}

	BlockModel->NumberOfModel++;
	return 1;
}

int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *CurPos, BLOCK_CELL *NextPos, char value)
{
	int8 row, col, i;

	col = CurPos->x;
	row = CurPos->y;
	for(i = 0 ; i < BlockSize; i++) {
		if((Block[i].x+col) > G_MaxCol || (Block[i].x+col) < 1) break;
		if((Block[i].y+row) > G_MaxRow || (Block[i].y+row) < 1) break;
		if(G_Matrix[Block[i].x+col][Block[i].y+row] != 0) break;
	}
	if(i >= BlockSize) {
		//OK can insert it
		for(i = 0 ; i < BlockSize; i++) {
			G_Matrix[Block[i].x+col][Block[i].y+row] = value;
		}
		//Find Next Pos
		for(;row <= G_MaxRow; row++) {
			for(;col <= G_MaxCol; col++) {
				if(G_Matrix[col][row] == 0) {
					NextPos->x = col;
					NextPos->y = row;
					return (1);
				}
			}
			col = 1;
		}
		NextPos->x = -1;
		NextPos->y = -1;
		return (1);
	}

	return (0);
}

void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos)
{
	int i;

	for(i = 0 ; i < BlockSize; i++) {
		G_Matrix[Block[i].x+InsertPos->x][Block[i].y+InsertPos->y] = 0;
	}
}


void PrintMatrix(char *startstring)
{
	int8 row, col;
	int8 value, stop;

	for(row = 0; row <= G_MaxRow ; row++) {
		stop = 1;
		printf("%s", startstring);
		for(col = 0; col <= G_MaxCol; col++) {
			value = 0;
			if(G_Matrix[col][row] != G_Matrix[col+1][row]) value += 1;
			if(G_Matrix[col][row] != G_Matrix[col][row+1]) value += 2;
			if(G_Matrix[col][row+1] != G_Matrix[col+1][row+1]) value += 4;
			if(G_Matrix[col+1][row] != G_Matrix[col+1][row+1]) value += 8;
			printf("%c", TanFlag[value]);
			if(value >= 9) printf("");
			else printf(" ");
			if(value) {
				stop = 0;
			}
		}
		printf("\n");
		if(stop) break;
	}
}

/*
void PrintMatrix(char *startstring)
{
	int8 row, col;

	for(row = 1; row <= G_MaxRow ; row++) {
		printf("%s", startstring);
		for(col = 1; col <= G_MaxCol; col++) {
			printf("%02d ", G_Matrix[col][row]);
		}
		printf("\n");
	}
	printf("\n");
}
*/

void TryMatrix(int HeadItem, BLOCK_CELL *CurPos)
{
	BLOCK_CELL NextPos;
	int        CurItem;
	int        i;

	CurItem = HeadItem;
	do {
		for(i = 0 ; i < G_BlockModel[CurItem].NumberOfModel; i++) {
			if(InsertToMatrix(G_BlockModel[CurItem].Block[i],
			                  G_BlockModel[CurItem].BlockSize,
			                  CurPos, &NextPos, CurItem+1))
			{

				G_BlockModel[CurItem].BlockInUsed = 1;
				G_BlockInUsed++;
				if((AllBlockInUsed()) || (NextPos.x == -1 && NextPos.y == -1)) {
					//(1) All block has put into matrix
					//(2) Matrix has fill blocks
					G_TotalAnswer++;
					printf("(%d)\n", G_TotalAnswer);
					PrintMatrix("");
				} else {
					TryMatrix(FindNextAvailableBlock(CurItem), &NextPos);
				}
				DeleteFromMatrix(G_BlockModel[CurItem].Block[i],
				                 G_BlockModel[CurItem].BlockSize,
				                 CurPos);
				G_BlockModel[CurItem].BlockInUsed = 0;
				G_BlockInUsed--;
			}
		}
		CurItem = FindNextAvailableBlock(CurItem);
	} while (CurItem != HeadItem);

	//printf("Total Answer = %d \n", G_TotalAnswer);
}

int8 FindNextAvailableBlock(int8 CurItem)
{
	int8 i;

	for(i = 0 ; i < G_TotalBlocks; i++) {
		if(++CurItem >= G_TotalBlocks) CurItem = 0;
		if(!G_BlockModel[CurItem].BlockInUsed) return (CurItem);
	}
	while(1) printf("FindNextAvailableBlock(): Desgined Error no any available block found !\n");
}
