#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <signal.h>
#include <fcntl.h>
#include <io.h>
#include <unistd.h>
#include <sys/exceptn.h>

typedef signed char int8;

typedef struct BlockCell {
	int8 x;
	int8 y;
	int8 z;
} BLOCK_CELL;

typedef struct blockmodel {
	int8 NumberOfModel;
	int8 BlockSize;
	int8 BlockInUsed;
	BLOCK_CELL *Block[24];
} BLOCK_MODEL;

#define AllBlockInUsed()  (G_BlockInUsed >= G_TotalBlocks)

void Usage(char *FName);
int  ReadMatrixData(FILE *fp);
void ShowMatrixData(void);
void Show3ViewMap(BLOCK_CELL *Block, int8 BlockSize);
void ShowPos(int x, int y);
void SkipSpace(char	*buf, int *Pos);
void SkipDigital(char *buf, int *Pos);
int  SkipTillDigital(char *buf, int *Pos);
void CreateBlockModel(void);
void PrintBlockModel(BLOCK_MODEL *BlockModel);
void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize);
int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock);
int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize,BLOCK_CELL *CurPos, BLOCK_CELL *NextPos, char value);
void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos);
void TryMatrix(int HeadItem, BLOCK_CELL *CurPos);
int8 FindNextAvailableBlock(int8 CurItem);

void PrintBig3DMatrix(char *startstring, int8 ***Array,BLOCK_CELL *pos);
void Print3DMatrix(char *startstring, int8 ***Array,BLOCK_CELL *pos);
void Print2DMatrix(int8 high, char *startstring);

void Print3D3ViewMap(void);
void PrintBig3DViewMatrix(int8 ***Array,int8 *XSize,int YSize, int8 ZSize);
void Print3DViewMatrix(int8 ***Array,int8 *XSize,int YSize, int8 ZSize);

void ShowCurBlock(BLOCK_CELL *CurBlock,int8 BlockSize, char ch);

int8 G_MaxRow;
int8 G_MaxCol;
int  G_MaxHig;
int8 ***G_Matrix;  //(Z, X, Y)

int8 G_BlockInUsed;
int8 G_TotalBlocks;

int  G_AnswerMatch;  //0 : all answer
int  G_TotalAnswer;

int  G_ShowBig5;
time_t G_start_t, G_end_t;

BLOCK_MODEL *G_BlockModel;

BLOCK_CELL RotateUnit[] = { {1, 1, 1} , {1, -1, -1}, {-1, 1, -1}, {-1, -1, 1} };

//        ͻ
//            1    
//        28͹
//            4    
//        ͼ

//      (3)    (5)    (6)      (7)     (9)  	  (10)
//        ͼ             ͻ    ͹               
//                                  
//
//                (12)         (13)         (14)          (15)
//     (11)                                                     
//                                   
//                                                             
//


//                  0    1    2    3	4    5    6    7
char TanFlag[] = { ' ', 'X', 'X', '', 'X', '', '', '',
//                  8    9   10   11   12   13   14   15
                   'X', '', '', '', '', '', '', ''  };

//                     0	 1     2     3     4     5     6     7
char *CHTanFlag[] = { "  ", "XX", "XX", "}", "XX", "x", "{", "t",
//                     8	 9	   10	 11	   12	 13	   14	 15
                      "XX", "|", "w", "r", "z", "u", "s", "q" };

void EndProg(int x)
{
    G_end_t = time(0);

	printf("\n******************************\n");

	if(G_AnswerMatch > 0) {
		printf("* Try to find %d answer ! \n",G_AnswerMatch);
	} else {
		printf("* Try to find all answer !\n");
    }
	printf("* Total found answer = %d !\n",G_TotalAnswer);
    printf("* Total Time = %d sec.\n",G_end_t - G_start_t);
	printf("******************************");
	if (x != 0) {
		printf("\n!!! Break by user !!!\n");
	    exit(1);
	}
}

int main(int argc, char *argv[])
{
	FILE *fp = NULL;
	BLOCK_CELL StartPos;
    int i, UsageOK;
    int ShowMatrixDataOnly = 0;

	signal(SIGINT, EndProg);

	if(argc < 2) {
	    Usage(argv[0]);
	    return (1);
	}

    UsageOK = 1;

	for(i = 1 ; i < argc; i++) {
		if(argv[i][0] == '-' && (argv[i][1] == 'N' || argv[i][1] == 'n')) {
			if( (G_AnswerMatch = atoi(&argv[i][2])) <= 0) {
				printf("-n<Number> : Number can not less or equal to zero !\n");
				UsageOK = 0;
				break;
			}
		} else
		if(!stricmp(argv[i],"-D")) {
			ShowMatrixDataOnly = 1;
		} else
		if(!stricmp(argv[i], "-BIG5")) {
			G_ShowBig5 = 1;
		} else {
			if(argv[i][0] == '-') {
				printf("Unknow option : %s !\n",argv[i]);
				UsageOK = 0;
				break;
			}
			if(fp != NULL) {
				printf("Too many files : %s !\n",argv[i]);
				UsageOK = 0;
				break;
			}
			if( (fp = fopen(argv[i], "rt")) == NULL) {
					printf("Open File: %s Error !\n",argv[i]);
					UsageOK = 0;
					break;
			}
        }
	}

    if(!UsageOK || fp == NULL) {
		Usage(argv[0]);
		return(2);
    }

	if(ReadMatrixData(fp)) {
            //Parse Matrix Data Error !
	        fclose(fp);
	        return (3);
	}
    if(ShowMatrixDataOnly) {
		ShowMatrixData();
        return (0);
	}

	CreateBlockModel();
    StartPos.x = StartPos.y = StartPos.z = 1;
	G_start_t = time(0);
	TryMatrix(0, &StartPos);

	EndProg(0);

	return (0);
}

void Usage(char *FName)
{
	printf("\nUsage: %s [-n<number>|-d] [-big5] <Matrix Data File> !\n\n", strrchr(FName,'/')+1);
	printf("         option : -n<number> : Display <number> answer only \n");
	printf("         option : -d : Dump matrix data only \n");
	printf("         option : -big5 : Use BIG5 code to draw table !\n");
	printf("\n         <Matrix Data File>: a file contain below format !\n");
	printf("         Matrix Data Format:\n");
	printf("             //Matrix Size (X Y Z) \n");
	printf("             3 4 1\n");
	printf("             //How many Blocks: \n");
	printf("             2\n");
	printf("             //Each Block: BlockSize, X1 Y1 Z1, X2 Y2 Z2, X3 Y3 Z3....\n");
	printf("             5, 0 0 0, 1 0 0, 2 0 0, 3 0 0, 0 1 0\n");
	printf("             4, 0 0 0, 0 1 0, 0 2 0, 0 3 0\n\n");
}

int ReadMatrixData(FILE *fp)
{
	char buf[256];
	int  ReadStep = 0;
	int  CurPos = 0;
	int8 CurNumOfBlock = 0;
	int  LineNumber = 0;
	int8 CurNumOfCell;
        int  i, j, BlockSize;

	while(fgets(buf,255,fp) != NULL) {
		CurPos = 0;
		LineNumber++;
		SkipSpace(buf,&CurPos);
		if(buf[CurPos] == '/' || buf[CurPos] == '\0') continue;
		if(!isdigit(buf[CurPos])) {
			printf("ReadMatrixData(): Syntax Error");
			ShowPos(LineNumber,CurPos+1);
			return (99);
		}
		switch(ReadStep) {
		case 0:	//Read Matrix Size
			G_MaxCol = atoi(buf+CurPos);
			SkipDigital(buf,&CurPos);
			if(!SkipTillDigital(buf, &CurPos)) {
				printf("ReadMatrixData():  Parse Matrix Size Error");
				ShowPos(LineNumber,CurPos+1);
				return (1);
			}
			G_MaxRow = atoi(buf+CurPos);
			SkipDigital(buf,&CurPos);
			if(!SkipTillDigital(buf, &CurPos)) {
				printf("ReadMatrixData():  Parse Matrix Size Error");
				ShowPos(LineNumber,CurPos+1);
				return (1);
			}
			G_MaxHig = atoi(buf+CurPos);

			// (Z)
			if(!(G_Matrix = (int8 ***) malloc((G_MaxHig+2) * sizeof(int8 **)))) {
				printf("ReadMatrixData(): Alloc Matrix Array Fail (Z)");
				ShowPos(LineNumber,CurPos+1);
				return (2);
			}
			// (X)
			for(i = 0 ; i < (G_MaxHig+2); i++) {
				if(!(G_Matrix[i] = (int8 **) calloc((G_MaxCol+2) , sizeof(int8 *)))) {
					printf("ReadMatrixData(): Alloc Matrix Array Fail (X)");
					ShowPos(LineNumber,CurPos+1);
					return (3);
				}
			}

			// (Y)
			for(i = 0 ; i < (G_MaxHig+2); i++) {
				for(j = 0 ; j < (G_MaxCol+2); j++) {
					if(!(G_Matrix[i][j] = (int8 *) calloc((G_MaxRow+2) , sizeof(int8)))) {
						printf("ReadMatrixData(): Alloc Matrix Array Fail (Y)");
						ShowPos(LineNumber,CurPos+1);
						return (3);
					}
				}
			}

			ReadStep++;
			break;
		case 1: //Read Total Blocks
			G_BlockInUsed =  G_TotalAnswer = 0;

			G_TotalBlocks = atoi(buf+CurPos);
			if(!(G_BlockModel = (BLOCK_MODEL *) calloc(G_TotalBlocks , sizeof(BLOCK_MODEL)))) {
				printf("ReadMatrixData(): Alloc Total Blocks Fail");
				ShowPos(LineNumber,CurPos+1);
				return (4);
			}
			ReadStep++;
			break;
		case 2:	//Read Block Cell for each Block

			//Read BlockSize
			if( (BlockSize = atoi(buf+CurPos)) <= 0) {
				printf("ReadMatrixData(): Wrong Block %d Size", CurNumOfBlock+1);
				ShowPos(LineNumber,CurPos+1);
				return (6);
			}

			//Alloc BlockSize
			G_BlockModel[CurNumOfBlock].NumberOfModel = 1;
			G_BlockModel[CurNumOfBlock].BlockSize = BlockSize;
			G_BlockModel[CurNumOfBlock].BlockInUsed = 0;

			G_BlockModel[CurNumOfBlock].Block[0] = (BLOCK_CELL *) calloc(BlockSize , sizeof(BLOCK_CELL));
			if(!G_BlockModel[CurNumOfBlock].Block[0]) {
				printf("ReadMatrixData(): Alloc Block %d Fail", CurNumOfBlock);
				ShowPos(LineNumber,CurPos+1);
				return (7);
			}

			CurNumOfCell = 0;
			do {

				SkipDigital(buf,&CurPos);
				//Skip until digital (,)
				if(!SkipTillDigital(buf, &CurPos)) {
					printf("ReadMatrixData(): Syntax Error (Block %d . X%d)",CurNumOfBlock+1, CurNumOfCell+1);
					ShowPos(LineNumber,CurPos+1);
					return (8);
				}

				//Read Block Cell X
				G_BlockModel[CurNumOfBlock].Block[0][CurNumOfCell].x = atoi(buf+CurPos);

				SkipDigital(buf,&CurPos);

				//Skip Space
				SkipSpace(buf,&CurPos);
				if(!isdigit(buf[CurPos]) && buf[CurPos] != '-') {
					printf("ReadMatrixData(): Syntax Error (Block %d . Y%d)",CurNumOfBlock+1, CurNumOfCell+1);
					ShowPos(LineNumber,CurPos+1);
					return (10);
				}

				//Read Block Cell Y
				G_BlockModel[CurNumOfBlock].Block[0][CurNumOfCell].y = atoi(buf+CurPos);

				SkipDigital(buf,&CurPos);

				//Skip Space
				SkipSpace(buf,&CurPos);
				if(!isdigit(buf[CurPos]) && buf[CurPos] != '-') {
					printf("ReadMatrixData(): Syntax Error (Block %d . Z%d)",CurNumOfBlock+1, CurNumOfCell+1);
					ShowPos(LineNumber,CurPos+1);
					return (10);
				}

				//Read Block Cell Z
				G_BlockModel[CurNumOfBlock].Block[0][CurNumOfCell].z = atoi(buf+CurPos);

				CurNumOfCell++;

			} while(CurNumOfCell < BlockSize);
			if(++CurNumOfBlock >= G_TotalBlocks) ReadStep++;
			break;
		case 3:
		default:
			printf("ReadMatrixData(): Too many Blocks");
			ShowPos(LineNumber,CurPos+1);
			return (12);
		}
	}// while (....)
	if(ReadStep != 3) {
		printf("ReadMatrixData(): Too few data");
		ShowPos(LineNumber+1,1);
		return (15);
	}
	return (0); //OK
}

void ShowPos(int x, int y)
{
	printf(" at Line %d Column %d\n", x,y);
}

void SkipSpace(char	*buf, int *Pos)
{
	while(buf[*Pos] == ' ' || buf[*Pos] == '\t' || buf[*Pos] == '\n') (*Pos)++;
}

void SkipDigital(char *buf, int *Pos)
{
	while(isdigit(buf[*Pos]) || buf[*Pos] == '-') (*Pos)++;
}

int SkipTillDigital(char *buf, int *Pos)
{
	while(buf[*Pos] != '-' && !isdigit(buf[*Pos]) && buf[*Pos] != '\0') (*Pos)++;

	return (isdigit(buf[*Pos]) || (buf[*Pos] == '-'));
}

void ShowMatrixData(void)
{
	int i, k;

	printf("\n//Matrix Size (X Y Z) \n");
	printf("%d %d %d\n\n", G_MaxCol, G_MaxRow, G_MaxHig);
	printf("//How many Blocks: \n");
	printf("%d\n\n",G_TotalBlocks);
	printf("//Each Block: BlockSize, X1 Y1 Z1, X2 Y2 Z2, X3 Y3 ....\n");
	for(i = 0 ; i < G_TotalBlocks; i++) {
		printf("\n//Block %d :\n", i+1);
		Show3ViewMap(G_BlockModel[i].Block[0],G_BlockModel[i].BlockSize);
		printf("%d",G_BlockModel[i].BlockSize);
		for(k =  0 ; k < G_BlockModel[i].BlockSize; k++) {
			printf(", %d %d %d", G_BlockModel[i].Block[0][k].x,
			                     G_BlockModel[i].Block[0][k].y,
			                     G_BlockModel[i].Block[0][k].z
			);
		}
		printf("\n");
	}
}

//							  Z	view       X view        Y view
BLOCK_CELL MAP3DUnit[] = { {3, 2, -1} , {-1, 2, -3} , {2, 1, -3} };

void Show3ViewMap(BLOCK_CELL *Block, int8 BlockSize)
{
	BLOCK_CELL *TmpBlock;
	BLOCK_CELL TmpBlockCell;
	int8 TmpBuf[3];
	int8 **Array[3]; //(Z, X, Y)  //Array[0] no used
	int i , j , k;

	if(!(TmpBlock = (BLOCK_CELL *) malloc((BlockSize) * sizeof(BLOCK_CELL)))) {
		printf("Alloc memory for 3D view Map Fail (1) !\n");
		exit(1);
	}

	for(i = 1 ; i <= 2; i++) {
		if(!(Array[i] = (int8 **) malloc((BlockSize+2) * sizeof(int8 *)))) {
			printf("Alloc memory for 3D view Map Fail (2) !\n");
			exit(1);
		}

		for(j = 0; j < (BlockSize+2); j++) {
			if(!(Array[i][j] = (int8 *) calloc((BlockSize+2) , sizeof(int8)))) {
				printf("Alloc memory for 3D view Map Fail (3) !\n");
				exit(1);
			}
		}
	}


	for(i = 0 ; i < 3; i++) {
		for(j = 0 ; j < BlockSize; j++) {
			TmpBuf[0] = Block[j].x;
			TmpBuf[1] = Block[j].y;
			TmpBuf[2] = Block[j].z;

			TmpBlock[j].x = TmpBuf[abs(MAP3DUnit[i].x)-1] * (MAP3DUnit[i].x > 0 ? 1 :-1);
			TmpBlock[j].y = TmpBuf[abs(MAP3DUnit[i].y)-1] * (MAP3DUnit[i].y > 0 ? 1 :-1);
			TmpBlock[j].z = TmpBuf[abs(MAP3DUnit[i].z)-1] * (MAP3DUnit[i].z > 0 ? 1 :-1);
		}

		//Sort it and get smallest value (sort by X from big ==> small )
		TmpBuf[0] =	TmpBlock[0].x;
		TmpBuf[1] = TmpBlock[0].y;
		TmpBuf[2] = TmpBlock[0].z;
		for(j = 0 ; j < BlockSize; j++) {
			for(k = j+1 ; k < BlockSize; k++) {
				if(TmpBlock[j].x < TmpBlock[k].x) {
					//swap it
					TmpBlockCell.x = TmpBlock[j].x;
					TmpBlockCell.y = TmpBlock[j].y;
					TmpBlockCell.z = TmpBlock[j].z;

					TmpBlock[j].x = TmpBlock[k].x;
					TmpBlock[j].y = TmpBlock[k].y;
					TmpBlock[j].z = TmpBlock[k].z;

					TmpBlock[k].x = TmpBlockCell.x;
					TmpBlock[k].y = TmpBlockCell.y;
					TmpBlock[k].z = TmpBlockCell.z;
				}
			}
			//Keep each smallest value
			if(TmpBlock[j].x < TmpBuf[0]) TmpBuf[0] = TmpBlock[j].x;
			if(TmpBlock[j].y < TmpBuf[1]) TmpBuf[1] = TmpBlock[j].y;
			if(TmpBlock[j].z < TmpBuf[2]) TmpBuf[2] = TmpBlock[j].z;
		}

		//Normalization (keep as puls)
		for(j = 0 ; j <	BlockSize; j++) {
			TmpBlock[j].x -= TmpBuf[0];
			TmpBlock[j].y -= TmpBuf[1];
			TmpBlock[j].z -= TmpBuf[2];
		}

		//Draw 3D Map
		switch(i) {
		case 0:	//(1) put z View into Array[1] and draw it
		case 1:	//(2) put x View into Array[1] and don't draw it
			//put z View into Array[1]
			for(j = 0; j < BlockSize; j++) {
				if(Array[1][TmpBlock[j].y+1][TmpBlock[j].z+1] ==0)
					Array[1][TmpBlock[j].y+1][TmpBlock[j].z+1] = TmpBlock[j].x+1;
			}
			if(i == 0) {
				TmpBlockCell.x = TmpBlockCell.y = BlockSize;
				TmpBlockCell.z = 1;
				Print3DMatrix("//  ", Array,&TmpBlockCell);
				for(j = 0; j < BlockSize; j++) {
					Array[1][TmpBlock[j].y+1][TmpBlock[j].z+1] = 0;
                }
			}
			break;
		case 2:	//(3) put y View into Array[2] and draw X and Y view
			for(j = 0; j < BlockSize; j++) {
				if(Array[2][TmpBlock[j].y+1][TmpBlock[j].z+1] ==0)
					Array[2][TmpBlock[j].y+1][TmpBlock[j].z+1] = TmpBlock[j].x+1;
			}
			TmpBlockCell.x = TmpBlockCell.y = BlockSize;
			TmpBlockCell.z = 2;
			Print3DMatrix("//  ", Array,&TmpBlockCell);
			break;
		default:
			printf("Show3ViewMap() Designed Error !\n");
			exit(1);
			break;
		}
	}

	free(TmpBlock);
	for(i = 1 ; i <= 2; i++) {
		for(j = 0; j < (BlockSize+2); j++) free(Array[i][j]);
		free(Array[i]);
	}
}

							  //{1, 2, 3}
BLOCK_CELL _3D_BasicUnit[] = { {-1, 3, 2}, {-2, 1, 3},
						      {2, 3, 1}, {3, 1, 2}, {-3, 2, 1}};
							 //{1, 1, 1}
BLOCK_CELL _3D_RotateUnit[] = { {1,  -1, -1}, {-1, 1, -1}, {-1, -1,  1} };

void CreateBlockModel(void)
{
	int        item;
	BLOCK_CELL *CurBlock;
	int8       i, j, k;
    int8       NumOfModel, NumOfBasicUnit;

	int8 TmpBuf[3];

	for(item = 0 ; item < G_TotalBlocks; item++) {
		CurBlock = (BLOCK_CELL *) malloc(G_BlockModel[item].BlockSize * sizeof(BLOCK_CELL));
		if(CurBlock == NULL) {
			printf("CreateBlockModel(): Alloc memory for Rotate Block Fail !\n");
			exit(1);
		}

		ShowCurBlock(G_BlockModel[item].Block[0],G_BlockModel[item].BlockSize,'O');

		for(i = 0 ; i < (int8) (sizeof(_3D_BasicUnit)/sizeof(BLOCK_CELL)); i++) {
			for(j = 0 ; j < G_BlockModel[item].BlockSize; j++) {
	        	NumOfModel = G_BlockModel[item].NumberOfModel - 1;

				TmpBuf[0] = G_BlockModel[item].Block[0][j].x;
				TmpBuf[1] = G_BlockModel[item].Block[0][j].y;
				TmpBuf[2] = G_BlockModel[item].Block[0][j].z;

				CurBlock[j].x = TmpBuf[abs(_3D_BasicUnit[i].x)-1] * (_3D_BasicUnit[i].x > 0 ? 1 :-1);
				CurBlock[j].y = TmpBuf[abs(_3D_BasicUnit[i].y)-1] * (_3D_BasicUnit[i].y > 0 ? 1 :-1);
				CurBlock[j].z = TmpBuf[abs(_3D_BasicUnit[i].z)-1] * (_3D_BasicUnit[i].z > 0 ? 1 :-1);
			}

			BlockNormalization(CurBlock, G_BlockModel[item].BlockSize);
			InsertBlock(G_BlockModel+item,CurBlock);
		}

		NumOfBasicUnit =  G_BlockModel[item].NumberOfModel;

		for(i = 0 ; i < NumOfBasicUnit; i++) {
			for(j = 0 ; j <	(int8)(sizeof(_3D_RotateUnit)/sizeof(BLOCK_CELL)); j++) {
				for(k = 0 ; k < G_BlockModel[item].BlockSize; k++) {
					CurBlock[k].x = G_BlockModel[item].Block[i][k].x * _3D_RotateUnit[j].x;
					CurBlock[k].y = G_BlockModel[item].Block[i][k].y * _3D_RotateUnit[j].y;
					CurBlock[k].z = G_BlockModel[item].Block[i][k].z * _3D_RotateUnit[j].z;
				}
				BlockNormalization(CurBlock, G_BlockModel[item].BlockSize);
				InsertBlock(G_BlockModel+item, CurBlock);
			}
		}

		free(CurBlock);

//		printf("\n\n");

//		for(i = 0 ; i < G_BlockModel[item].NumberOfModel; i++) {
//			ShowMatrix(G_BlockModel[item].Block[i],G_BlockModel[item].BlockSize);
//		}
	}
}


void BlockCopy(BLOCK_CELL *SrcBlock, BLOCK_CELL *TagBlock, int8 BlockSize)
{
	int8 i;

	for(i = 0; i < BlockSize; i++) {
		TagBlock[i].x = SrcBlock[i].x;
		TagBlock[i].y = SrcBlock[i].y;
		TagBlock[i].z = SrcBlock[i].z;
	}
}

void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize)
{
	BLOCK_CELL TmpBlockCell;
	int8 i, j;

	//Sort it  (compare Z then Y then X)
	for(i = 0 ; i < (BlockSize-1); i++) {
		for(j = i+1; j < BlockSize; j++) {
			if( (Block[j].z < Block[i].z) ||
				(Block[j].z == Block[i].z && Block[j].y < Block[i].y) ||
				(Block[j].z == Block[i].z && Block[j].y == Block[i].y && Block[j].x < Block[i].x))
			{
				//swapt it
				TmpBlockCell.x = Block[j].x;
				TmpBlockCell.y = Block[j].y;
				TmpBlockCell.z = Block[j].z;

				Block[j].x = Block[i].x;
				Block[j].y = Block[i].y;
				Block[j].z = Block[i].z;

				Block[i].x = TmpBlockCell.x;
				Block[i].y = TmpBlockCell.y;
				Block[i].z = TmpBlockCell.z;
			}
		}
	}

	//Set smallest cell as (0,0)

	TmpBlockCell.x = Block[0].x;
	TmpBlockCell.y = Block[0].y;
	TmpBlockCell.z = Block[0].z;

	for(i = 0; i < BlockSize; i++) {
		Block[i].x -=  TmpBlockCell.x;
		Block[i].y -=  TmpBlockCell.y;
		Block[i].z -=  TmpBlockCell.z;
//		printf("(%d, %d, %d) ",Block[i].x,Block[i].y, Block[i].z);
	}
//	printf("\n");


}

int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock)
{
	int8 i, j;

	for(i = 0 ; i < BlockModel->NumberOfModel ; i++) {
		for(j = 0 ; j < BlockModel->BlockSize; j++) {
			if((BlockModel->Block[i][j].x != CurBlock[j].x) ||
			   (BlockModel->Block[i][j].y != CurBlock[j].y) ||
			   (BlockModel->Block[i][j].z != CurBlock[j].z)	)
			{
				break;
			}
		}
		if( j >= BlockModel->BlockSize) {
			ShowCurBlock(CurBlock,BlockModel->BlockSize,'X');
			return 0;
		}
	}

	BlockModel->Block[BlockModel->NumberOfModel] = (BLOCK_CELL *) malloc(BlockModel->BlockSize * sizeof(BLOCK_CELL));
	if(BlockModel->Block[BlockModel->NumberOfModel] == NULL) {
		printf("InsertBlock(): Alloc memory Fail !\n");
		exit(1);
	}
	for(i = 0 ; i < BlockModel->BlockSize; i++) {
		BlockModel->Block[BlockModel->NumberOfModel][i].x = CurBlock[i].x;
		BlockModel->Block[BlockModel->NumberOfModel][i].y = CurBlock[i].y;
		BlockModel->Block[BlockModel->NumberOfModel][i].z = CurBlock[i].z;
	}

	BlockModel->NumberOfModel++;
	ShowCurBlock(CurBlock,BlockModel->BlockSize,'O');
	return 1;
}

void ShowCurBlock(BLOCK_CELL *CurBlock,int8 BlockSize, char ch)
{
/*
	int i;

	printf("(%c) ",ch);
	for(i = 0 ; i < BlockSize; i++) {
		printf(", (%02d, %02d, %02d)", CurBlock[i].x, CurBlock[i].y, CurBlock[i].z);
    }
	printf("\n");
*/
}


int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *CurPos, BLOCK_CELL *NextPos, char value)
{
	int8 row, col, hig, i;

	col = CurPos->x;
	row = CurPos->y;
	hig = CurPos->z;
	for(i = 0 ; i < BlockSize; i++) {
		if((Block[i].x+col) > G_MaxCol || (Block[i].x+col) < 1) break;
		if((Block[i].y+row) > G_MaxRow || (Block[i].y+row) < 1) break;
		if((Block[i].z+hig) > G_MaxHig || (Block[i].z+hig) < 1) break;
		if(G_Matrix[Block[i].z+hig][Block[i].x+col][Block[i].y+row] != 0) break;
	}
	if(i >= BlockSize) {
		//OK can insert it
		for(i = 0 ; i < BlockSize; i++) {
			G_Matrix[Block[i].z+hig][Block[i].x+col][Block[i].y+row] = value;
		}
		//Find Next Pos
		for(;hig <= G_MaxHig; hig++) {
			for(;row <= G_MaxRow; row++) {
				for(;col <= G_MaxCol; col++) {
					if(G_Matrix[hig][col][row] == 0) {
						NextPos->x = col;
						NextPos->y = row;
						NextPos->z = hig;
//						Print2DMatrix(hig,"");
						return (1);
					}
				}
				col = 1;
			}
			row = 1;
		}
		NextPos->x = -1;
		NextPos->y = -1;
		NextPos->z = -1;
		return (1);
	}

	return (0);
}

void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos)
{
	int i;

	for(i = 0 ; i < BlockSize; i++) {
		G_Matrix[Block[i].z+InsertPos->z][Block[i].x+InsertPos->x][Block[i].y+InsertPos->y] = 0;
	}
}

void PrintBig3DMatrix(char *startstring, int8 ***Array,BLOCK_CELL *pos)
{
	int8 x, y, z;
	int8 value, stop;
	int8 duple;

	for(y = 0; y <= pos->y ; y++) {
		stop = 1;
		printf("%s", startstring);
		for(duple = 0 ; duple < ((y < pos->y) ? 2:1) ; duple++) {
		for(z = 1 ; z <= pos->z ; z++) {
			for(x = 0 ; x <= pos->x; x++) {
				value = 0;
				if(Array[z][x][y] != Array[z][x+1][y]) value += 1;
				if(Array[z][x][y] != Array[z][x][y+1]) value += 2;
				if(Array[z][x][y+1] != Array[z][x+1][y+1]) value += 4;
				if(Array[z][x+1][y] != Array[z][x+1][y+1]) value += 8;
				if(G_ShowBig5) {
					if(duple == 0) {
						printf("%s", CHTanFlag[value]);
						if(value >= 9) printf("w");
						else printf("  ");
					} else {
						if(value & 4) {
							printf("x");
						} else printf("  ");
						if(x < pos->x) printf("%2d",Array[z][x+1][y+1]);
						else printf("  ");
					}
				} else {
					if(duple == 0) {
						printf("%c", TanFlag[value]);
						if(value >= 9) printf("");
						else printf("   ");
					} else {
						if(y < pos->y) {
							if(value & 4) {
								printf("");
							} else printf(" ");
							if(x < pos->x ) printf("%2d ",Array[z][x+1][y+1]);
							else printf("   ");
						}
					}
				}
				if(value) stop = 0;
			}
		}
		printf("\n");
		}
		if(stop) break;
	}
}

void Print3DMatrix(char *startstring, int8 ***Array,BLOCK_CELL *pos)
{
	int8 x, y, z;
	int8 value, stop;

	for(y = 0; y <= pos->y ; y++) {
		stop = 1;
		printf("%s", startstring);
		for(z = 1 ; z <= pos->z ; z++) {
			for(x = 0 ; x <= pos->x; x++) {
				value = 0;
				if(Array[z][x][y] != Array[z][x+1][y]) value += 1;
				if(Array[z][x][y] != Array[z][x][y+1]) value += 2;
				if(Array[z][x][y+1] != Array[z][x+1][y+1]) value += 4;
				if(Array[z][x+1][y] != Array[z][x+1][y+1]) value += 8;
				if(G_ShowBig5) {
					printf("%s", CHTanFlag[value]);
					if(value >= 9) printf("w");
					else printf("  ");
				} else {
					printf("%c", TanFlag[value]);
					if(value >= 9) printf("");
					else printf(" ");
				}
				if(value) stop = 0;
			}
		}
		printf("\n");
		if(stop) break;
	}
}

void PrintBig3DViewMatrix(int8 ***Array,int8 *XSize,int YSize, int8 ZSize)
{
	int8 x, y, z;
	int8 value, stop;
	int8 duple;

	for(y = 0; y <= YSize ; y++) {
		stop = 1;
//		printf("%s", startstring);
		for(duple = 0 ; duple < ((y < YSize) ? 2 : 1); duple++) {
		for(z = 0 ; z < ZSize ; z++) {
			for(x = 0 ; x <= XSize[z]; x++) {
				value = 0;
				if(Array[z][x][y] != Array[z][x+1][y]) value += 1;
				if(Array[z][x][y] != Array[z][x][y+1]) value += 2;
				if(Array[z][x][y+1] != Array[z][x+1][y+1]) value += 4;
				if(Array[z][x+1][y] != Array[z][x+1][y+1]) value += 8;

				if(G_ShowBig5) {
					if(duple == 0) {
						printf("%s", CHTanFlag[value]);
						if(value >= 9) printf("w");
						else printf("  ");
					} else {
						if(value & 4) {
							printf("x");
						} else printf("  ");
						if(x < XSize[z] && y < YSize) printf("%2d",Array[z][x+1][y+1]);
						else printf("  ");
					}
				} else {
					if(duple == 0) {
						printf("%c", TanFlag[value]);
						if(value >= 9) printf("");
						else printf("   ");
					} else {
						if(value & 4) {
							printf("");
						} else printf(" ");
						if(x < XSize[z]) printf("%2d ",Array[z][x+1][y+1]);
						else printf("   ");
					}
				}
				if(value) stop = 0;
			}
		}
		printf("\n");
		}
		if(stop) break;
	}
}

void Print3DViewMatrix(int8 ***Array,int8 *XSize,int YSize, int8 ZSize)
{
	int8 x, y, z;
	int8 value, stop;

	for(y = 0; y <= YSize ; y++) {
		stop = 1;
//		printf("%s", startstring);
		for(z = 0 ; z < ZSize ; z++) {
			for(x = 0 ; x <= XSize[z]; x++) {
				value = 0;
				if(Array[z][x][y] != Array[z][x+1][y]) value += 1;
				if(Array[z][x][y] != Array[z][x][y+1]) value += 2;
				if(Array[z][x][y+1] != Array[z][x+1][y+1]) value += 4;
				if(Array[z][x+1][y] != Array[z][x+1][y+1]) value += 8;
				if(G_ShowBig5) {
					printf("%s", CHTanFlag[value]);
					if(value >= 9) printf("w");
					else printf("  ");
				} else {
					printf("%c", TanFlag[value]);
					if(value >= 9) printf("");
					else printf(" ");
				}
				if(value) stop = 0;
			}
		}
		printf("\n");
		if(stop) break;
	}
}


void Print2DMatrix(int8 high, char *startstring)
{
	int8 row, col;
	int8 value, stop;

	for(row = 0; row <= G_MaxRow ; row++) {
		stop = 1;
		printf("%s", startstring);
		for(col = 0; col <= G_MaxCol; col++) {
			value = 0;
			if(G_Matrix[high][col][row] != G_Matrix[high][col+1][row]) value += 1;
			if(G_Matrix[high][col][row] != G_Matrix[high][col][row+1]) value += 2;
			if(G_Matrix[high][col][row+1] != G_Matrix[high][col+1][row+1]) value += 4;
			if(G_Matrix[high][col+1][row] != G_Matrix[high][col+1][row+1]) value += 8;

			if(G_ShowBig5) {
				printf("%c", TanFlag[value]);
				if(value >= 9) printf("w");
				else printf("  ");
			} else {
				printf("%c", TanFlag[value]);
				if(value >= 9) printf("");
				else printf(" ");
			}
			if(value) stop = 0;
		}
		printf("\n");
		if(stop) break;
	}
}


              //   3: Z,  2:Y,  1:X
              //   Y   X  Z     Y   Z   X    X   Z  Y
int8 XYZ[][3] = { {2, -1, 3},  {2, -3, -1}, {1, -3, 2},
              //   Y   Z  X     X   Z   Y    Y  X   Z
                  {-2, -3, 1}, {-1, -3, -2}, {2, 1, -3} };

void Print3D3ViewMap(void)
{
#define XXvalue (Value[0])
#define YYvalue (Value[1])
#define ZZvalue (Value[2])

	int8 MatrixSize[3]; //X, Y Z size
	int8 Xvalue, Yvalue;
	int8 Value[3], *P[3];
	int8 **_3DViewMatrix[6];
	int8 _3DViewXSize[6];
	int8 _3DViewYSize[6];
        int8 i , j, k, item;
        int8 Punit[2];

	MatrixSize[0] = G_MaxCol;	 //X
	MatrixSize[1] = G_MaxRow;	 //Y
	MatrixSize[2] = G_MaxHig;	 //Z

	//Alloc memory for put matrix value
	for(i = 0 ; i < 6; i++) {
		Xvalue = MatrixSize[abs(XYZ[i][0])-1]+2;
		Yvalue = MatrixSize[abs(XYZ[i][1])-1]+2;

		_3DViewMatrix[i] = (int8 **) malloc(Xvalue * sizeof(int8 *));
		for(j = 0 ; j < Xvalue ; j++) {
			_3DViewMatrix[i][j] = (int8 *) calloc(Yvalue , sizeof(int8));
		}
	}

	//
	for(item = 0 ; item < 6; item++) {
		XXvalue = _3DViewXSize[item] = MatrixSize[abs(XYZ[item][0])-1];	 //X
		YYvalue = _3DViewYSize[item] = MatrixSize[abs(XYZ[item][1])-1];	 //Y
		ZZvalue = MatrixSize[abs(XYZ[item][2])-1];

		if(XYZ[item][0]	> 0) {
			XXvalue  = 1;
			Punit[0] = 1;
		} else Punit[0] = -1;

		if(XYZ[item][1]	> 0) {
			YYvalue  = 1;
			Punit[1] = 1;
		} else Punit[1] = -1;

		if(XYZ[item][2] < 0) ZZvalue = 1;

		Xvalue = XXvalue;
		Yvalue = YYvalue;

		for(i = 0 ; i < 3; i++) {
			for(j = 0 ; j < 3; j++) {
				if(i == abs(XYZ[item][j])-1) {
				    P[i] = &Value[j];
				    break;
				}
			}
		}

		for(j = 1, XXvalue = Xvalue; j <= _3DViewXSize[item]; j++, XXvalue += Punit[0]) {
			for(k = 1, YYvalue = Yvalue; k <= _3DViewYSize[item]; k++, YYvalue += Punit[1]) {
				//            Z	   X  Y				Z	  X		Y
				_3DViewMatrix[item][j][k] = G_Matrix[*P[2]][*P[0]][*P[1]];
			}
		}
	}

//	Print3DViewMatrix(_3DViewMatrix,&_3DViewXSize[0],_3DViewYSize[0], 1);
//	Print3DViewMatrix(&_3DViewMatrix[1],&_3DViewXSize[1],_3DViewYSize[1], 4);
//	Print3DViewMatrix(&_3DViewMatrix[5],&_3DViewXSize[5],_3DViewYSize[5], 1);

	PrintBig3DViewMatrix(_3DViewMatrix,&_3DViewXSize[0],_3DViewYSize[0], 1);
	PrintBig3DViewMatrix(&_3DViewMatrix[1],&_3DViewXSize[1],_3DViewYSize[1], 4);
	PrintBig3DViewMatrix(&_3DViewMatrix[5],&_3DViewXSize[5],_3DViewYSize[5], 1);

	for(i = 0 ; i < 6; i++) {
		Xvalue = MatrixSize[abs(XYZ[i][0])-1]+2;

		for(j = 0 ; j < Xvalue ; j++) {
			free(_3DViewMatrix[i][j]);
		}
		free(_3DViewMatrix[i]);
	}
}

/*
void PrintMatrix(char *startstring)
{
	int8 row, col;

	for(row = 1; row <= G_MaxRow ; row++) {
		printf("%s", startstring);
		for(col = 1; col <= G_MaxCol; col++) {
			printf("%02d ", G_Matrix[col][row]);
		}
		printf("\n");
	}
	printf("\n");
}
*/

void TryMatrix(int HeadItem, BLOCK_CELL *CurPos)
{
	BLOCK_CELL NextPos;
	int        CurItem;
	int        i;

	CurItem = HeadItem;
	do {
		for(i = 0 ; i < G_BlockModel[CurItem].NumberOfModel; i++) {
			if(InsertToMatrix(G_BlockModel[CurItem].Block[i],
			                  G_BlockModel[CurItem].BlockSize,
			                  CurPos, &NextPos, CurItem+1))
			{

				G_BlockModel[CurItem].BlockInUsed = 1;
				G_BlockInUsed++;
				if((AllBlockInUsed()) || (NextPos.x == -1 && NextPos.y == -1)) {
					//(1) All block has put into matrix
					//(2) Matrix has fill blocks
					if(++G_TotalAnswer == 1) {
//							printf("***** Begin *****\n\n");
					}
					printf("(%d)\n\n\n\n\n\n\n\n\n\n\n\n", G_TotalAnswer);
					if(G_MaxHig == 1) Print2DMatrix(1,"");
					else {
						BLOCK_CELL P;
						P.x = G_MaxCol;
						P.y = G_MaxRow;
						P.z = G_MaxHig;
//						Print3DMatrix("",G_Matrix,&P);
						PrintBig3DMatrix("",G_Matrix,&P);
						printf("\n\n\n\n\n\n\n\n\n\n\n\n");
						Print3D3ViewMap();
					}
				} else {
					TryMatrix(FindNextAvailableBlock(CurItem), &NextPos);
				}
				DeleteFromMatrix(G_BlockModel[CurItem].Block[i],
				                 G_BlockModel[CurItem].BlockSize,
				                 CurPos);
				G_BlockModel[CurItem].BlockInUsed = 0;
				G_BlockInUsed--;
			}
		}
		CurItem = FindNextAvailableBlock(CurItem);
	} while (CurItem != HeadItem && (!G_AnswerMatch?1:G_TotalAnswer < G_AnswerMatch));
}

int8 FindNextAvailableBlock(int8 CurItem)
{
	int8 i;

	for(i = 0 ; i < G_TotalBlocks; i++) {
		if(++CurItem >= G_TotalBlocks) CurItem = 0;
		if(!G_BlockModel[CurItem].BlockInUsed) return (CurItem);
	}
	while(1) printf("FindNextAvailableBlock(): Desgined Error no any available block found !\n");
}
