#include <stdio.h>
#include <alloc.h>
#include <stdlib.h>

typedef signed char int8;

#define MAX_ROW   10
#define MAX_COL   10

typedef struct BlockCell {
	int8 x;
	int8 y;
} BLOCK_CELL;

typedef struct blockmodel {
	int8 NumberOfModel;
	int8 BlockSize;
	BLOCK_CELL *Block[8];
} BLOCK_MODEL;

void CreateBlockModel(BLOCK_MODEL *BlockModel);
void PrintBlockModel(BLOCK_MODEL *BlockModel);
void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize);
int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock);
int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos);
void PrintMatrix(void);
void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos);


int8 Matrix[MAX_ROW+2][MAX_COL+2];

BLOCK_CELL RotateUnit[] = { {1, -1} , {-1, 1}, {-1, -1} };

//		  ͻ
//		      1    
//        28͹
//            4    
//        ͼ

//      (3)    (5)    (6)      (7)     (9)  	  (10)
//        ͼ             ͻ    ͹               
//                                  
//
//                (12)         (13)         (14)          (15)
//     (11)                                                     
//       		                     
//                                                             
//


//                  0    1    2    3	4	 5	  6	   7
char TanFlag[] = { ' ', 'X', 'X', '', 'X', '', '', '',
//                  8    9   10   11   12   13   14   15
                   'X', '', '', '', '', '', '', ''  };



void main(void)
{
//	BLOCK_CELL OrgBlock[] = { {0,0}, {1,0}, {2,0}, {0,1}, {2,1}, {0,2} };
//	BLOCK_CELL OrgBlock[] = { {1,0}, {0,1}, {1,1}, {2,1}};
//	BLOCK_CELL OrgBlock[] = { {0,0}, {1,0}, {-1,1}, {0,1}, {-1,2} };
	BLOCK_CELL OrgBlock[] = { {0,0}, {0,1}, {1,1}, {2,1}, {0,2} };
	BLOCK_MODEL BlockModel;
	int8   i, BlockSize;

	BlockSize = BlockModel.BlockSize = sizeof(OrgBlock)/ sizeof(BLOCK_CELL);
	BlockModel.Block[0] = (BLOCK_CELL *) malloc(BlockSize * sizeof(BLOCK_CELL));

	for(i = 0; i < BlockSize; i++) {
		(BlockModel.Block[0])[i].x = OrgBlock[i].x;
		(BlockModel.Block[0])[i].y = OrgBlock[i].y;
	}
	BlockModel.NumberOfModel = 1;

	CreateBlockModel(&BlockModel);
	PrintBlockModel(&BlockModel);
}

void CreateBlockModel(BLOCK_MODEL *BlockModel)
{
//	int8       TmpCell;
	BLOCK_CELL *CurBlock;
	int8	   RotateTimes, i, j, k;

	CurBlock = (BLOCK_CELL *) malloc(BlockModel->BlockSize * sizeof(BLOCK_CELL));

	//(X , Y) ==> (Y , X)
	for(i = 0 ; i < BlockModel->BlockSize; i++) {
		CurBlock[i].x = BlockModel->Block[0][i].y;
		CurBlock[i].y = BlockModel->Block[0][i].x;
	}
	BlockNormalization(CurBlock, BlockModel->BlockSize);
	RotateTimes = (InsertBlock(BlockModel, CurBlock)) ? 2 : 1;

	for(i = 0 ; i < RotateTimes; i++) {

		//Insert Rotate Block !
		for(j = 0 ; j < sizeof(RotateUnit) / sizeof(BLOCK_CELL); j++) {
			for(k = 0 ; k < BlockModel->BlockSize; k++) {
				CurBlock[k].x = BlockModel->Block[i][k].x * RotateUnit[j].x;
				CurBlock[k].y = BlockModel->Block[i][k].y * RotateUnit[j].y;
			}
			BlockNormalization(CurBlock, BlockModel->BlockSize);
			InsertBlock(BlockModel, CurBlock);
		}
	}
	free(CurBlock);
}

void BlockCopy(BLOCK_CELL *SrcBlock, BLOCK_CELL *TagBlock, int8 BlockSize)
{
	int8 i;

	for(i = 0; i < BlockSize; i++) {
		TagBlock[i].x = SrcBlock[i].x;
		TagBlock[i].y = SrcBlock[i].y;
	}
}

void BlockNormalization(BLOCK_CELL *Block, int8 BlockSize)
{
	BLOCK_CELL TmpBlockCell;
	int8 i, j;

	//Sort it  (compare Y then X)
	for(i = 0 ; i < (BlockSize-1); i++) {
		for(j = i+1; j < BlockSize; j++) {
			if( (Block[j].y < Block[i].y) ||
				(Block[j].y == Block[i].y  && Block[j].x < Block[i].x) )
			{
            	//swapt it
				TmpBlockCell.x = Block[j].x;
				TmpBlockCell.y = Block[j].y;

				Block[j].x = Block[i].x;
				Block[j].y = Block[i].y;

				Block[i].x = TmpBlockCell.x;
				Block[i].y = TmpBlockCell.y;

			}
		}
	}

	//Set smallest cell as (0,0)

	TmpBlockCell.x = Block[0].x;
	TmpBlockCell.y = Block[0].y;

	for(i = 0; i < BlockSize; i++) {
		Block[i].x -=  TmpBlockCell.x;
		Block[i].y -=  TmpBlockCell.y;
//		printf("(%d, %d) ",Block[i].x,Block[i].y);
	}
//	printf("\n");

}

int8 InsertBlock(BLOCK_MODEL *BlockModel, BLOCK_CELL *CurBlock)
{
	int8 i, j;

	for(i = 0 ; i < BlockModel->NumberOfModel ; i++) {
		for(j = 0 ; j < BlockModel->BlockSize; j++) {
			if((BlockModel->Block[i])[j].x != CurBlock[j].x ||
			   (BlockModel->Block[i])[j].y != CurBlock[j].y)
			{
				break;
			}
		}
		if( j >= BlockModel->BlockSize) {
			return 0;
		}
	}

	BlockModel->Block[BlockModel->NumberOfModel] = (BLOCK_CELL *) malloc(BlockModel->BlockSize * sizeof(BLOCK_CELL));
	for(i = 0 ; i < BlockModel->BlockSize; i++) {
		BlockModel->Block[BlockModel->NumberOfModel][i].x = CurBlock[i].x;
		BlockModel->Block[BlockModel->NumberOfModel][i].y = CurBlock[i].y;
	}

	BlockModel->NumberOfModel++;
	return 1;
}

void PrintBlockModel(BLOCK_MODEL *BlockModel)
{
	BLOCK_CELL InsertPos;
	int8 i;

	for(i = 0 ; i < BlockModel->NumberOfModel; i++) {
		InsertToMatrix(BlockModel->Block[i], BlockModel->BlockSize, &InsertPos);
		PrintMatrix();
		DeleteFromMatrix(BlockModel->Block[i], BlockModel->BlockSize, &InsertPos);
	}
}

int8 InsertToMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos)
{
	int8 row, col, i;
	int8 FindPos = 0;

	for(row = 1; row <= MAX_ROW && !FindPos; row++) {
		for(col = 1; col <= MAX_COL && !FindPos; col++) {
			for(i = 0 ; i < BlockSize; i++) {
				if((Block[i].x+col) > MAX_ROW || (Block[i].x+col) < 1) break;
				if((Block[i].y+row) > MAX_COL || (Block[i].y+row) < 1) break;
				if(Matrix[Block[i].x+col][Block[i].y+row] == 1) break;
			}
			if(i >= BlockSize) {
			    FindPos = 1;
			    InsertPos->x = col;
			    InsertPos->y = row;
			}
		}
	}
	if(FindPos) {
		for(i = 0 ; i < BlockSize; i++) {
			Matrix[Block[i].x+InsertPos->x][Block[i].y+InsertPos->y] = 1;
		}
	}

	return (FindPos);
}

void DeleteFromMatrix(BLOCK_CELL *Block,int8 BlockSize, BLOCK_CELL *InsertPos)
{
	int i;

	for(i = 0 ; i < BlockSize; i++) {
		Matrix[Block[i].x+InsertPos->x][Block[i].y+InsertPos->y] = 0;
	}
}


void PrintMatrix(void)
{
	int8 row, col;
	int8 value, stop;

	for(row = 0; row <= MAX_ROW ; row++) {
		stop = 1;
		for(col = 0; col <= MAX_COL; col++) {
			value = 0;
			if(Matrix[col][row] != Matrix[col+1][row]) value += 1;
			if(Matrix[col][row] != Matrix[col][row+1]) value += 2;
			if(Matrix[col][row+1] != Matrix[col+1][row+1]) value += 4;
			if(Matrix[col+1][row] != Matrix[col+1][row+1]) value += 8;
			printf("%c",TanFlag[value]);
			if(value) stop = 0;
		}
		printf("\n");
		if(stop) break;
	}
}

